
# q-expansion of newform 7200.2.k.b, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7200
weight = 2
poly_data = [2, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-1, 2], 1]]

hecke_ring_character_values = [[6751, [1, 0]], [901, [-1, 0]], [6401, [1, 0]], [577, [1, 0]]]
aps_data = [[0, 0], [0, 0], [0, 0], [-4, 0], [0, -1], [0, 0], [3, 0], [0, 1], [4, 0], [0, 0], [-4, 0], [0, 4], [5, 0], [0, -2], [-8, 0], [0, 4], [0, -2], [0, -4], [0, 3], [8, 0], [-7, 0], [-4, 0], [0, -3], [1, 0], [-2, 0], [0, -4], [-8, 0], [0, 1], [0, -4], [15, 0], [-12, 0], [0, 6], [19, 0], [0, -7], [0, 0], [-4, 0], [0, -4], [0, -5], [0, 0], [0, -8], [0, -9], [0, -4], [4, 0], [5, 0], [0, 4], [24, 0], [0, -3], [16, 0], [0, -6], [0, -8], [-6, 0], [-8, 0], [-21, 0], [0, 3], [-14, 0], [12, 0], [0, -8], [20, 0], [0, 8], [22, 0], [0, -5], [0, -4], [0, -1], [-4, 0], [6, 0], [0, 0], [0, 1], [-15, 0], [0, 1], [0, 4], [18, 0], [36, 0], [0, 0], [0, -4], [0, -3], [-36, 0], [0, 4], [0, 8], [-27, 0], [3, 0], [0, 7], [0, -8], [12, 0], [37, 0], [-8, 0], [0, -11], [-27, 0], [-27, 0], [0, -16], [8, 0], [0, 10], [4, 0], [12, 0], [0, -2], [0, 10], [0, 0], [0, -12], [-3, 0], [0, -1], [0, 8], [0, 7], [0, 0], [0, -6], [-11, 0], [0, -14], [-7, 0], [0, -3], [-41, 0], [-12, 0], [7, 0], [8, 0], [0, 4], [-6, 0], [0, 2], [-36, 0], [-2, 0], [0, 6], [0, 0], [0, -12], [0, 3], [0, -8], [-34, 0], [0, -16], [0, 9], [0, 9], [0, 8], [0, -8], [36, 0], [16, 0], [0, 0], [0, -6], [36, 0], [-8, 0], [0, -8], [29, 0], [-21, 0], [0, 0], [0, -10], [0, 12], [-10, 0], [0, 2], [0, -4], [24, 0], [0, 5], [0, 8], [40, 0], [0, -20], [-21, 0], [0, 1], [24, 0], [0, -16], [30, 0], [0, -17], [56, 0], [0, -2], [48, 0], [-4, 0], [14, 0], [-7, 0], [0, 16], [0, -6], [-5, 0], [-8, 0], [0, -9], [-37, 0], [-28, 0], [-44, 0], [0, -16], [43, 0], [0, 0], [0, 11], [0, -4], [-36, 0], [-47, 0], [32, 0], [-23, 0], [0, 13], [0, -12], [0, 0], [0, 20], [8, 0], [0, -1], [0, -4], [7, 0], [16, 0], [0, -16], [0, -12], [0, 15], [-53, 0], [56, 0], [49, 0], [0, 21], [0, -11], [0, -12], [0, -15], [42, 0], [-25, 0], [0, 4], [63, 0], [28, 0], [0, 12], [16, 0], [0, 8], [-25, 0], [0, -2], [0, 12], [-20, 0], [0, -6], [6, 0], [0, 18], [-18, 0], [0, 12], [-8, 0], [0, -7], [60, 0], [-13, 0], [-68, 0], [21, 0], [36, 0], [0, -12], [0, 4], [-16, 0], [-3, 0], [-60, 0], [0, -15], [0, 12], [15, 0], [-72, 0], [48, 0], [0, -21], [0, 4], [0, 21], [20, 0], [-47, 0], [0, 14], [-60, 0], [35, 0], [0, 0], [0, 27], [40, 0], [0, 25], [0, -27], [28, 0], [0, 24], [-33, 0], [-48, 0], [-44, 0], [0, 6], [0, 5], [-8, 0], [0, 24], [49, 0], [-44, 0], [63, 0], [0, 24], [0, 19], [0, -8], [0, 3], [0, -4], [29, 0], [4, 0], [0, 5], [0, 16], [0, 8], [11, 0], [0, 25], [0, 20], [-42, 0], [0, 15], [0, -20], [0, 8], [0, -5], [81, 0], [-24, 0], [14, 0], [-64, 0], [0, 25], [0, 8], [-38, 0], [0, 15], [-24, 0], [12, 0], [72, 0], [0, -24], [0, -21], [24, 0], [30, 0], [0, 8], [-56, 0], [30, 0], [0, -20], [0, -22], [39, 0], [0, 19], [0, -32], [0, 12], [28, 0], [0, 24], [0, 14], [0, -29], [29, 0], [0, -12], [40, 0], [0, -7], [0, 5], [29, 0], [0, -30], [0, 4], [36, 0], [0, -4], [-36, 0], [0, 12], [69, 0], [0, -10], [48, 0], [-70, 0], [0, -9], [72, 0], [-47, 0], [57, 0], [0, 10], [13, 0], [0, -32], [68, 0], [-54, 0], [-14, 0], [0, 5], [0, 14], [-52, 0], [0, -24], [0, 24], [0, -20], [-44, 0], [0, -7], [0, 25], [0, -30], [0, -32], [34, 0], [7, 0], [36, 0], [0, -8], [-13, 0], [0, 12], [-40, 0], [0, 12], [0, 6], [0, 16], [0, 35], [0, 0], [0, 4], [0, 5], [86, 0], [0, 36], [88, 0], [0, 12], [23, 0], [-64, 0], [0, -25], [66, 0], [-72, 0], [0, -20], [49, 0], [-12, 0], [0, -18], [0, -5], [1, 0], [0, 8], [40, 0], [75, 0], [0, -26], [0, 17], [24, 0], [0, -32], [-32, 0], [0, -4], [0, 23], [-68, 0], [25, 0], [69, 0], [-58, 0], [0, -4], [-37, 0], [-16, 0], [39, 0], [0, -22], [-4, 0], [40, 0], [0, 16], [0, -2], [-12, 0], [-41, 0], [0, 24], [0, 3], [0, 3], [-60, 0], [-19, 0], [0, 0], [77, 0], [0, -7], [0, 4], [0, -8], [99, 0], [-80, 0], [-21, 0], [0, -24], [-48, 0], [0, 20], [-50, 0], [0, -10], [0, 19], [-7, 0], [0, -16], [0, -3], [0, -6], [1, 0], [0, 12], [20, 0], [-4, 0], [-93, 0], [84, 0], [0, 32], [0, 28], [-12, 0], [0, 3], [-75, 0], [0, 8], [0, 26], [33, 0], [0, -23], [60, 0]]
